#define S_FUNCTION_NAME  synchronizer
#define S_FUNCTION_LEVEL 2

#include "simstruc.h"
#include "matlab.h"
#include <time.h>
#include <windows.h>
//=====================================================
static clock_t StartTime;

#define DISCRETE_STEP 0.1

#define SYNCHR_R_IDX 0
#define SYNCHR_R_PARAM(S) ssGetSFcnParam(S,SYNCHR_R_IDX)
//=====================================================
//     S-function
//   1  -  
static void mdlCheckParameters(SimStruct *S)
{ const mxArray *para = (const mxArray *) SYNCHR_R_PARAM(S);
  real_T *N= mxGetPr(para);
  if ((mxGetM(SYNCHR_R_PARAM(S)) > 1) || 
     (mxGetN(SYNCHR_R_PARAM(S)) > 1)) {
        ssSetErrorStatus(S,"Parameter to S-function must be a scalar.");
       return;
  }
  if (N[0]<=0){ ssSetErrorStatus(S,"Parameter to S-function must be positive!");  return;
  }
}
//=====================================================
//  S-function
static void mdlInitializeSizes(SimStruct *S)
{ ssSetNumSFcnParams(S, 1);  
#if defined(MATLAB_MEX_FILE)
  if (ssGetNumSFcnParams(S) == ssGetSFcnParamsCount(S)) {
     mdlCheckParameters(S);
     if (ssGetErrorStatus(S) != NULL) return;
  } else  return;
#endif
  ssSetNumContStates( S, 0); 
  ssSetNumDiscStates( S, 0); 
  if (!ssSetNumInputPorts(S, 0)) return;
  if (!ssSetNumOutputPorts(S, 0)) return;
  ssSetNumSampleTimes( S, 1);
  ssSetOptions( S, 0); 
}
//=====================================================
//    S-function 
static void mdlInitializeSampleTimes(SimStruct *S)
{
  ssSetSampleTime(S, 0, DISCRETE_STEP);
  ssSetOffsetTime(S, 0, 0.0);
}
//=====================================================
//,      
static void mdlStart(SimStruct *S)
{ StartTime=clock();
}
//=====================================================
//   S-function
static void mdlOutputs(SimStruct *S, int_T tid)
{ const mxArray *para = (const mxArray *) SYNCHR_R_PARAM(S);
  double *N= mxGetPr(para);
  clock_t NowTime;
  DWORD diff_M_PC;
  real_T InTime=(real_T)ssGetT(S); //  
  long Tr_Hold = 1000*DISCRETE_STEP/N[0];

  NowTime=clock();
  diff_M_PC=NowTime-StartTime;
  if ((diff_M_PC>=0) && (diff_M_PC<Tr_Hold))
     Sleep(Tr_Hold-diff_M_PC);
  StartTime=clock();
}
//=====================================================
static void mdlTerminate(SimStruct *S)
{}
//=====================================================
#ifdef  MATLAB_MEX_FILE 
#include "simulink.c"   
#else
#include "cg_sfun.h" 
#endif
